@@ -265,7 +265,7 @@ update_output_settings (GvcMixerDialog *dialog)
                 GvcMixerStreamPort *port;
                 port = gvc_mixer_stream_get_port (stream);
 
-                dialog->priv->output_port_combo = gvc_combo_box_new (_("Connector:"));
+                dialog->priv->output_port_combo = gvc_combo_box_new (_("Co_nnector:"));
                 gvc_combo_box_set_ports (GVC_COMBO_BOX (dialog->priv->output_port_combo),
                                          ports);
                 gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->output_port_combo), port->port);
@@ -275,7 +275,7 @@ update_output_settings (GvcMixerDialog *dialog)
 
                 gtk_box_pack_start (GTK_BOX (dialog->priv->output_settings_box),
                                     dialog->priv->output_port_combo,
-                                    FALSE, FALSE, 12);
+                                    TRUE, FALSE, 0);
                 gtk_widget_show (dialog->priv->output_port_combo);
         }
 
@@ -436,10 +436,15 @@ create_monitor_stream_for_source (GvcMixerDialog *dialog,
         pa_context    *context;
         int            res;
         pa_proplist   *proplist;
+        gboolean       has_monitor;
 
         if (stream == NULL) {
                 return;
         }
+        has_monitor = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (stream), "has-monitor"));
+        if (has_monitor != FALSE) {
+                return;
+        }
 
         g_debug ("Create monitor for %u",
                  gvc_mixer_stream_get_index (stream));
@@ -481,9 +486,58 @@ create_monitor_stream_for_source (GvcMixerDialog *dialog,
         if (res < 0) {
                 g_warning ("Failed to connect monitoring stream");
                 pa_stream_unref (s);
+        } else {
+                g_object_set_data (G_OBJECT (stream), "has-monitor", GINT_TO_POINTER (TRUE));
         }
 }
 
+static void
+update_input_settings (GvcMixerDialog *dialog)
+{
+        const GList *ports;
+        GvcMixerStream *stream;
+
+        g_debug ("Updating input settings");
+
+        if (dialog->priv->input_port_combo != NULL) {
+                gtk_container_remove (GTK_CONTAINER (dialog->priv->input_settings_box),
+                                      dialog->priv->input_port_combo);
+                dialog->priv->input_port_combo = NULL;
+        }
+
+        stream = gvc_mixer_control_get_default_source (dialog->priv->mixer_control);
+        if (stream == NULL) {
+                g_warning ("Default source stream not found");
+                return;
+        }
+
+        gvc_channel_bar_set_base_volume (GVC_CHANNEL_BAR (dialog->priv->input_bar),
+                                         gvc_mixer_stream_get_base_volume (stream));
+        gvc_channel_bar_set_is_amplified (GVC_CHANNEL_BAR (dialog->priv->input_bar),
+                                          gvc_mixer_stream_get_can_decibel (stream));
+
+        ports = gvc_mixer_stream_get_ports (stream);
+        if (ports != NULL) {
+                GvcMixerStreamPort *port;
+                port = gvc_mixer_stream_get_port (stream);
+
+                dialog->priv->input_port_combo = gvc_combo_box_new (_("Co_nnector:"));
+                gvc_combo_box_set_ports (GVC_COMBO_BOX (dialog->priv->input_port_combo),
+                                         ports);
+                gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->input_port_combo), port->port);
+                g_object_set_data (G_OBJECT (dialog->priv->input_port_combo), "stream", stream);
+                g_signal_connect (G_OBJECT (dialog->priv->input_port_combo), "changed",
+                                  G_CALLBACK (port_selection_changed), dialog);
+
+                gtk_box_pack_start (GTK_BOX (dialog->priv->input_settings_box),
+                                    dialog->priv->input_port_combo,
+                                    TRUE, TRUE, 0);
+                gtk_widget_show (dialog->priv->input_port_combo);
+        }
+
+        create_monitor_stream_for_source (dialog, stream);
+}
+
 static void
 on_mixer_control_default_source_changed (GvcMixerControl *control,
                                          guint            id,
@@ -494,20 +548,19 @@ on_mixer_control_default_source_changed (GvcMixerControl *control,
 
         g_debug ("GvcMixerDialog: default source changed: %u", id);
 
-        stream = gvc_mixer_control_lookup_stream_id (dialog->priv->mixer_control,
-                                                     id);
+        stream = gvc_mixer_control_lookup_stream_id (dialog->priv->mixer_control, id);
+
+        /* Disconnect the adj, otherwise it might change if is_amplified changes */
         adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (dialog->priv->input_bar)));
         g_signal_handlers_disconnect_by_func(adj, on_adjustment_value_changed, dialog);
+
         bar_set_stream (dialog, dialog->priv->input_bar, stream);
-        gvc_channel_bar_set_base_volume (GVC_CHANNEL_BAR (dialog->priv->input_bar),
-                                         gvc_mixer_stream_get_base_volume (stream));
-        gvc_channel_bar_set_is_amplified (GVC_CHANNEL_BAR (dialog->priv->input_bar),
-                                          gvc_mixer_stream_get_can_decibel (stream));
+        update_input_settings (dialog);
+
         g_signal_connect (adj,
                           "value-changed",
                           G_CALLBACK (on_adjustment_value_changed),
                           dialog);
-        create_monitor_stream_for_source (dialog, stream);
 
         update_default_input (dialog);
 }
@@ -593,13 +646,27 @@ static void
 on_adjustment_value_changed (GtkAdjustment  *adjustment,
                              GvcMixerDialog *dialog)
 {
-        gdouble         volume;
         GvcMixerStream *stream;
 
-        stream = g_object_get_data (G_OBJECT (adjustment), "gvc-mixer-dialog-stream"),
-        volume = gtk_adjustment_get_value (adjustment);
+        stream = g_object_get_data (G_OBJECT (adjustment), "gvc-mixer-dialog-stream");
         if (stream != NULL) {
-                gvc_mixer_stream_set_volume(stream, (pa_volume_t) volume );
+        	GObject *bar;
+                gdouble volume, rounded;
+                char *name;
+
+                volume = gtk_adjustment_get_value (adjustment);
+                rounded = round (volume);
+
+                bar = g_object_get_data (G_OBJECT (adjustment), "gvc-mixer-dialog-bar");
+                g_object_get (bar, "name", &name, NULL);
+                g_debug ("Setting stream volume %lf (rounded: %lf) for bar '%s'", volume, rounded, name);
+                g_free (name);
+
+                /* FIXME would need to do that in the balance bar really... */
+                gvc_mixer_stream_set_is_muted (stream, volume == 0);
+                /* Only push the volume if it's actually changed */
+                if (gvc_mixer_stream_set_volume(stream, (pa_volume_t) rounded) != FALSE)
+                        gvc_mixer_stream_push_volume (stream);
         }
 }
 
@@ -791,21 +858,39 @@ bar_set_stream (GvcMixerDialog *dialog,
                 GtkWidget      *bar,
                 GvcMixerStream *stream)
 {
-        GtkAdjustment *adj;
-        gboolean       is_muted;
+        GtkAdjustment  *adj;
+        GvcMixerStream *old_stream;
 
         g_assert (bar != NULL);
 
-        is_muted = FALSE;
+        old_stream = g_object_get_data (G_OBJECT (bar), "gvc-mixer-dialog-stream");
+        if (old_stream != NULL) {
+                char *name;
+
+                g_object_get (bar, "name", &name, NULL);
+                g_debug ("Disconnecting old stream '%s' from bar '%s'",
+                         gvc_mixer_stream_get_name (old_stream), name);
+                g_free (name);
+
+                g_signal_handlers_disconnect_by_func (old_stream, on_stream_is_muted_notify, dialog);
+                g_signal_handlers_disconnect_by_func (old_stream, on_stream_volume_notify, dialog);
+                g_signal_handlers_disconnect_by_func (old_stream, on_stream_port_notify, dialog);
+                g_hash_table_remove (dialog->priv->bars, GUINT_TO_POINTER (gvc_mixer_stream_get_id (old_stream)));
+        }
 
         gtk_widget_set_sensitive (bar, (stream != NULL));
 
         adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (bar)));
 
+        g_signal_handlers_disconnect_by_func (adj, on_adjustment_value_changed, dialog);
+
         g_object_set_data (G_OBJECT (bar), "gvc-mixer-dialog-stream", stream);
         g_object_set_data (G_OBJECT (adj), "gvc-mixer-dialog-stream", stream);
+        g_object_set_data (G_OBJECT (adj), "gvc-mixer-dialog-bar", bar);
 
         if (stream != NULL) {
+                gboolean is_muted;
+
                 is_muted = gvc_mixer_stream_get_is_muted (stream);
                 gvc_channel_bar_set_is_muted (GVC_CHANNEL_BAR (bar), is_muted);
 
@@ -813,6 +898,23 @@ bar_set_stream (GvcMixerDialog *dialog,
 
                 gtk_adjustment_set_value (adj,
                                           gvc_mixer_stream_get_volume (stream));
+
+                g_signal_connect (stream,
+                                  "notify::is-muted",
+                                  G_CALLBACK (on_stream_is_muted_notify),
+                                  dialog);
+                g_signal_connect (stream,
+                                  "notify::volume",
+                                  G_CALLBACK (on_stream_volume_notify),
+                                  dialog);
+                g_signal_connect (stream,
+                                  "notify::port",
+                                  G_CALLBACK (on_stream_port_notify),
+                                  dialog);
+                g_signal_connect (adj,
+                                  "value-changed",
+                                  G_CALLBACK (on_adjustment_value_changed),
+                                  dialog);
         }
 }
 
@@ -828,6 +930,9 @@ add_stream (GvcMixerDialog *dialog,
 
         g_assert (stream != NULL);
 
+        if (gvc_mixer_stream_is_event_stream (stream) != FALSE)
+                return;
+
         bar = NULL;
         is_default = FALSE;
         id = gvc_mixer_stream_get_application_id (stream);
@@ -838,59 +943,40 @@ add_stream (GvcMixerDialog *dialog,
                 is_default = TRUE;
                 gtk_widget_set_sensitive (dialog->priv->applications_box,
                                           !is_muted);
-
-                update_output_settings (dialog);
-        } else if (stream == gvc_mixer_control_get_default_source (dialog->priv->mixer_control)) {
-                const GList *ports;
-                bar = dialog->priv->input_bar;
-                is_default = TRUE;
-
-                /* Disconnect the adj, otherwise it might change if is_amplified changes */
                 adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (bar)));
                 g_signal_handlers_disconnect_by_func(adj, on_adjustment_value_changed, dialog);
-
-                gvc_channel_bar_set_base_volume (GVC_CHANNEL_BAR (dialog->priv->input_bar),
-                                                 gvc_mixer_stream_get_base_volume (stream));
-                gvc_channel_bar_set_is_amplified (GVC_CHANNEL_BAR (bar),
-                                                  gvc_mixer_stream_get_can_decibel (stream));
-
-                if (dialog->priv->input_port_combo != NULL) {
-                        gtk_container_remove (GTK_CONTAINER (dialog->priv->input_settings_box),
-                                              dialog->priv->input_port_combo);
-                        dialog->priv->input_port_combo = NULL;
-                }
-                ports = gvc_mixer_stream_get_ports (stream);
-                if (ports != NULL) {
-                        GvcMixerStreamPort *port;
-                        port = gvc_mixer_stream_get_port (stream);
-
-                        dialog->priv->input_port_combo = gvc_combo_box_new (_("Connector:"));
-                        gvc_combo_box_set_ports (GVC_COMBO_BOX (dialog->priv->input_port_combo),
-                                                 ports);
-                        gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->input_port_combo), port->port);
-                        g_object_set_data (G_OBJECT (dialog->priv->input_port_combo), "stream", stream);
-                        g_signal_connect (G_OBJECT (dialog->priv->input_port_combo), "changed",
-                                          G_CALLBACK (port_selection_changed), dialog);
-
-                        gtk_box_pack_start (GTK_BOX (dialog->priv->input_settings_box),
-                                            dialog->priv->input_port_combo,
-                                            TRUE, TRUE, 12);
-                        gtk_widget_show (dialog->priv->input_port_combo);
-                }
-
-                create_monitor_stream_for_source (dialog, stream);
+                update_output_settings (dialog);
+        } else if (stream == gvc_mixer_control_get_default_source (dialog->priv->mixer_control)) {
+                bar = dialog->priv->input_bar;
+                adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (bar)));
+                g_signal_handlers_disconnect_by_func(adj, on_adjustment_value_changed, dialog);
+                update_input_settings (dialog);
+                is_default = TRUE;
         } else if (stream == gvc_mixer_control_get_event_sink_input (dialog->priv->mixer_control)) {
                 bar = dialog->priv->effects_bar;
                 g_debug ("Adding effects stream");
         } else if (! GVC_IS_MIXER_SOURCE (stream)
                    && !GVC_IS_MIXER_SINK (stream)
-                   && !gvc_mixer_stream_is_event_stream (stream)
                    && !gvc_mixer_stream_is_virtual (stream)
                    && g_strcmp0 (id, "org.gnome.VolumeControl") != 0
                    && g_strcmp0 (id, "org.PulseAudio.pavucontrol") != 0) {
+                const char *name;
+
                 bar = create_bar (dialog, dialog->priv->apps_size_group, FALSE);
-                gvc_channel_bar_set_name (GVC_CHANNEL_BAR (bar),
-                                          gvc_mixer_stream_get_name (stream));
+
+                name = gvc_mixer_stream_get_name (stream);
+                if (name == NULL || strchr (name, '_') == NULL) {
+                        gvc_channel_bar_set_name (GVC_CHANNEL_BAR (bar), name);
+                } else {
+                        char **tokens, *escaped;
+
+                        tokens = g_strsplit (name, "_", -1); 
+                        escaped = g_strjoinv ("__", tokens);
+                        g_strfreev (tokens);
+                        gvc_channel_bar_set_name (GVC_CHANNEL_BAR (bar), escaped);
+                        g_free (escaped);
+                }
+
                 gvc_channel_bar_set_icon_name (GVC_CHANNEL_BAR (bar),
                                                gvc_mixer_stream_get_icon_name (stream));
 
@@ -932,27 +1018,8 @@ add_stream (GvcMixerDialog *dialog,
 
         if (bar != NULL) {
                 bar_set_stream (dialog, bar, stream);
-                adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (bar)));
-
-                g_signal_connect (adj,
-                                  "value-changed",
-                                  G_CALLBACK (on_adjustment_value_changed),
-                                  dialog);
                 gtk_widget_show (bar);
         }
-
-        g_signal_connect (stream,
-                          "notify::is-muted",
-                          G_CALLBACK (on_stream_is_muted_notify),
-                          dialog);
-        g_signal_connect (stream,
-                          "notify::volume",
-                          G_CALLBACK (on_stream_volume_notify),
-                          dialog);
-        g_signal_connect (stream,
-                          "notify::port",
-                          G_CALLBACK (on_stream_port_notify),
-                          dialog);
 }
 
 static void
@@ -1017,7 +1084,10 @@ remove_stream (GvcMixerDialog  *dialog,
         if (bar == dialog->priv->output_bar
             || bar == dialog->priv->input_bar
             || bar == dialog->priv->effects_bar) {
-                g_hash_table_remove (dialog->priv->bars, GUINT_TO_POINTER (id));
+                char *name;
+                g_object_get (bar, "name", &name, NULL);
+                g_debug ("Removing stream for bar '%s'", name);
+                g_free (name);
                 bar_set_stream (dialog, bar, NULL);
         } else if (bar != NULL) {
                 g_hash_table_remove (dialog->priv->bars, GUINT_TO_POINTER (id));
@@ -1360,7 +1430,7 @@ on_card_selection_changed (GtkTreeSelection *selection,
 
         current_profile = gvc_mixer_card_get_profile (card);
         profiles = gvc_mixer_card_get_profiles (card);
-        dialog->priv->hw_profile_combo = gvc_combo_box_new (_("Profile:"));
+        dialog->priv->hw_profile_combo = gvc_combo_box_new (_("_Profile:"));
         gvc_combo_box_set_profiles (GVC_COMBO_BOX (dialog->priv->hw_profile_combo), profiles);
         gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->hw_profile_combo), current_profile->profile);
 
@@ -1512,7 +1582,7 @@ gvc_mixer_dialog_constructor (GType                  type,
                             FALSE, FALSE, 12);
         self->priv->output_bar = create_bar (self, self->priv->size_group, TRUE);
         gvc_channel_bar_set_name (GVC_CHANNEL_BAR (self->priv->output_bar),
-                                  _("Output Volume: "));
+                                  _("_Output volume: "));
         gtk_widget_set_sensitive (self->priv->output_bar, FALSE);
         gtk_box_pack_start (GTK_BOX (self->priv->output_stream_box),
                             self->priv->output_bar, TRUE, TRUE, 12);
@@ -1549,7 +1619,7 @@ gvc_mixer_dialog_constructor (GType                  type,
 
         self->priv->effects_bar = create_bar (self, self->priv->size_group, TRUE);
         gvc_channel_bar_set_name (GVC_CHANNEL_BAR (self->priv->effects_bar),
-                                  _("Alert Volume: "));
+                                  _("_Alert volume: "));
         gtk_widget_set_sensitive (self->priv->effects_bar, FALSE);
         gtk_box_pack_start (GTK_BOX (self->priv->sound_effects_box),
                             self->priv->effects_bar, FALSE, FALSE, 12);
@@ -1567,9 +1637,10 @@ gvc_mixer_dialog_constructor (GType                  type,
                                   self->priv->hw_box,
                                   label);
 
-        box = gtk_frame_new (_("Choose a device to configure"));
+        box = gtk_frame_new (_("C_hoose a device to configure:"));
         label = gtk_frame_get_label_widget (GTK_FRAME (box));
         _gtk_label_make_bold (GTK_LABEL (label));
+        gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
         gtk_frame_set_shadow_type (GTK_FRAME (box), GTK_SHADOW_NONE);
         gtk_box_pack_start (GTK_BOX (self->priv->hw_box), box, TRUE, TRUE, 0);
 
@@ -1579,6 +1650,8 @@ gvc_mixer_dialog_constructor (GType                  type,
 
         self->priv->hw_treeview = create_cards_treeview (self,
                                                          G_CALLBACK (on_card_selection_changed));
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), self->priv->hw_treeview);
+
         box = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (box),
                                         GTK_POLICY_NEVER,
@@ -1609,7 +1682,7 @@ gvc_mixer_dialog_constructor (GType                  type,
 
         self->priv->input_bar = create_bar (self, self->priv->size_group, TRUE);
         gvc_channel_bar_set_name (GVC_CHANNEL_BAR (self->priv->input_bar),
-                                  _("Input Volume: "));
+                                  _("_Input volume: "));
         gvc_channel_bar_set_low_icon_name (GVC_CHANNEL_BAR (self->priv->input_bar),
                                            "audio-input-microphone-low");
         gvc_channel_bar_set_high_icon_name (GVC_CHANNEL_BAR (self->priv->input_bar),
@@ -1656,9 +1729,10 @@ gvc_mixer_dialog_constructor (GType                  type,
                             self->priv->input_settings_box,
                             FALSE, FALSE, 0);
 
-        box = gtk_frame_new (_("Choose a device for sound input"));
+        box = gtk_frame_new (_("C_hoose a device for sound input:"));
         label = gtk_frame_get_label_widget (GTK_FRAME (box));
         _gtk_label_make_bold (GTK_LABEL (label));
+        gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
         gtk_frame_set_shadow_type (GTK_FRAME (box), GTK_SHADOW_NONE);
         gtk_box_pack_start (GTK_BOX (self->priv->input_box), box, TRUE, TRUE, 0);
 
@@ -1668,6 +1742,8 @@ gvc_mixer_dialog_constructor (GType                  type,
 
         self->priv->input_treeview = create_stream_treeview (self,
                                                              G_CALLBACK (on_input_radio_toggled));
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), self->priv->input_treeview);
+
         box = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (box),
                                         GTK_POLICY_NEVER,
@@ -1688,9 +1764,10 @@ gvc_mixer_dialog_constructor (GType                  type,
                                   self->priv->output_box,
                                   label);
 
-        box = gtk_frame_new (_("Choose a device for sound output"));
+        box = gtk_frame_new (_("C_hoose a device for sound output:"));
         label = gtk_frame_get_label_widget (GTK_FRAME (box));
         _gtk_label_make_bold (GTK_LABEL (label));
+        gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
         gtk_frame_set_shadow_type (GTK_FRAME (box), GTK_SHADOW_NONE);
         gtk_box_pack_start (GTK_BOX (self->priv->output_box), box, TRUE, TRUE, 0);
 
@@ -1700,6 +1777,8 @@ gvc_mixer_dialog_constructor (GType                  type,
 
         self->priv->output_treeview = create_stream_treeview (self,
                                                               G_CALLBACK (on_output_radio_toggled));
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), self->priv->output_treeview);
+
         box = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (box),
                                         GTK_POLICY_NEVER,
